"""Logger interface"""


class Logger:
    def __init__(self, name=None):
        pass

    def update_logger(self,
                      log_level: int=None,
                      log_format=None,
                      log_path=None,
                      stream_out: bool=False,
                      coloredlog: bool=False,
                      **optionals):
        pass

    def debug(self, msg, *args, **kwargs):
        pass

    def info(self, msg, *args, **kwargs):
        pass

    def warning(self, msg, *args, **kwargs):
        pass

    def error(self, msg, *args, **kwargs):
        pass

    def exception(self, msg, *args, exc_info=True, **kwargs):
        pass

    def critical(self, msg, *args, **kwargs):
        pass



"""Logger"""
import logging
from typing import Optional
from logging.handlers import TimedRotatingFileHandler

import coloredlogs

coloredlogs.DEFAULT_LEVEL_STYLES = {
    'info': {},
    'notice': {'color': 'magenta'},
    'verbose': {'color': 'blue'},
    'success': {'color': 'green', 'bold': True},
    'spam': {'color': 'cyan'},
    'critical': {'color': 'red', 'bold': True},
    'error': {'color': 'red'},
    'debug': {'color': 'green'},
    'warning': {'color': 'yellow'}}


class LoggerBase:
    def __init__(self, name=None):
        self._logger = logging.getLogger(name or __name__)
        self._default_format = "%(asctime)s %(name)s [%(levelname)s] %(message)s"
        self.update_logger()

    def get_logger(self):
        return self._logger

    def __getattr__(self, name):
        try:
            return self._logger.__getattribute__(name)
        except AttributeError:
            return self.__getattribute__(name)

    def _reset_handlers(self):
        for handler in self._logger.handlers[:]:
            self._logger.removeHandler(handler)

    def update_logger(self,
                      log_level: Optional[int] = None,
                      log_format=None,
                      log_path=None,
                      stream_out: bool = False,
                      coloredlog: bool = False,
                      **optionals):
        _log_level = log_level or logging.DEBUG
        _log_format = log_format or self._default_format
        self._logger.setLevel(_log_level)
        formatter = logging.Formatter(_log_format)

        self._reset_handlers()

        if stream_out:
            console = logging.StreamHandler()
            console.setLevel(_log_level)
            console.setFormatter(formatter)
            self._logger.addHandler(console)

            if coloredlog:
                style = coloredlogs.DEFAULT_LEVEL_STYLES.copy()
                coloredlogs.install(reconfigure=True,
                                    level=logging.getLevelName(_log_level), milliseconds=True, logger=self._logger,
                                    fmt=_log_format,
                                    datefmt='%H:%M:%S',
                                    level_styles=style)

        if log_path:
            file_handler = logging.FileHandler(filename=log_path)

            if optionals:
                log_when = optionals.get('log_when')
                log_interval = optionals.get('log_interval')
                log_backup_count = optionals.get('log_backup_count')

                if None not in [log_when, log_interval, log_backup_count]:
                    file_handler = TimedRotatingFileHandler(
                        filename=log_path,
                        when=log_when,
                        interval=log_interval,
                        backupCount=log_backup_count
                    )

            file_handler.setLevel(_log_level)
            file_handler.setFormatter(formatter)
            self._logger.addHandler(file_handler)


class Logger(LoggerBase):
    pass



"""Test Config"""
import os
from pathlib import Path

import pytest

from yirgachefe import config as config_
from yirgachefe import logger
from yirgachefe.config_ import Config

logger.update_logger(stream_out=True, coloredlog=True)
config = config_


@pytest.fixture
def config_path() -> Path:
    root_path = Path(os.path.join(os.path.dirname(__file__), '.'))
    file_path: Path = root_path.joinpath('temp-configure.json')
    return file_path


@pytest.fixture(autouse=True)
def patch_config_with_params(config_path):
    from yirgachefe.config_ import Config
    global config

    file_path: Path = config_path
    if file_path.exists():
        file_path.unlink()

    config = Config(config_path=file_path, logger=logger)
    config.write_config(config_path=file_path)
    yield
    if file_path.exists():
        file_path.unlink()


class TestConfig:
    def test_get_config(self):
        assert config['debug'] is not None

    def test_load_config(self, config_path):
        config.load_config(config_path)
        assert config['debug'] is not None

    def test_default_config_path(self):
        config_with_default_path = Config()
        assert config_with_default_path._config_path == config_with_default_path.get_path(config.CONFIG_DEFAULT_FILE)

    def test_load_env(self):
        os.environ['DEBUG'] = 'True'
        config._load_config_env()
        assert config['debug'] is True

        os.environ['debug'] = 'False'
        config._load_config_env()
        assert config['debug'] is False

    def test_set_config_in_code(self):
        with pytest.raises(KeyError):
            print(config['NEW'])
        config['NEW'] = 'new value'
        assert config['NEW'] == 'new value'
        ori_debug_mode = config['debug']
        config['debug'] = not ori_debug_mode
        assert config['debug'] != ori_debug_mode

    def test_use_config_as_attributes(self):
        with pytest.raises(KeyError):
            print(config.NEW)
        config.NEW = 'new value'
        assert config['NEW'] == 'new value'
        ori_debug_mode = config.debug
        config.debug = not ori_debug_mode
        assert config.debug != ori_debug_mode

    def test_write_config(self, config_path):
        config.NEW = 'new2 value2'
        config.write_config(config_path)
        config2 = Config(config_path=config_path)
        assert config.NEW == config2['NEW']

        config3 = Config()
        config3.NEW = 'new3 value3'
        config3.write_config(config_path)
        config4 = Config(config_path=config_path)
        assert config3.NEW == config4.NEW



"""Config"""
import json
import logging
import os
from pathlib import Path
from typing import Optional

from yirgachefe.logger_ import Logger


class Config:
    CONFIG_DEFAULT_FILE = 'configure.json'

    def __init__(self, config_path: Optional[Path] = None, logger=None):
        super().__init__()
        self._current_path: Optional[Path] = None
        self._config_path = config_path or self.get_path(self.CONFIG_DEFAULT_FILE)
        self._config = {}
        self._logger: Logger = logger or logging.getLogger(__name__)
        self._log_options = None
        self._load_config_file(self._config_path, check_exist=False)
        self._set_common_default()
        self._load_config_env()
        self._update_log_options()

    def load_config(self, config_path: Path):
        """Load User configuration file.

        :param config_path: The str path created with pathlib.Path is recommended.
        """
        self._load_config_file(config_path=config_path)
        self._update_log_options()

    def __setattr__(self, key, value):
        if key in ['_current_path', '_config_path', '_config', '_logger', '_log_options']:
            self.__dict__[key] = value
        else:
            self._config[key] = value

    def __getattr__(self, name):
        if name in self.__dict__:
            return self.__getattribute__(name)
        return self[name]

    def __setitem__(self, key, item):
        self._config[key] = item
        self._update_log_options()

    def __getitem__(self, key):
        return self._config[key]

    def _set_common_default(self):
        if 'debug' not in self._config:
            self._logger.debug("config['debug'] is not configured. So default(False) is used.")
            self._config['debug'] = False
        if 'log_level' not in self._config:
            self._logger.debug("config['log_level'] is not configured. So default(logging.WARNING) is used.")
            self._config['log_level'] = logging.getLevelName(logging.WARNING)
        if 'log_format' not in self._config:
            self._logger.debug("config['log_format'] is not in configured. So default is used.")
            self._config['log_format'] = \
                "%(asctime)s,%(msecs)03d %(process)d %(thread)d %(levelname)s %(filename)s(%(lineno)d) %(message)s"

    def _load_config_file(self, config_path: Path, check_exist=True):
        if not check_exist and not config_path.is_file():
            return

        with open(config_path, 'r') as config_file:
            self._config.update(json.load(config_file))
            self._current_path = config_path.parent

    def _load_config_env(self):
        for config_name in self._config:
            env_value = os.getenv(config_name) or os.getenv(str.upper(config_name))
            if env_value is not None:
                if isinstance(self[config_name], (bool, int, float)):
                    env_value = eval(env_value)
                self[config_name] = env_value

    def _update_log_options(self):
        """Update log_options and logger."""
        prev_log_options = self._log_options
        self._log_options = ''.join(
            [str(self._config.get(key)) for key in ['debug', 'log_level', 'log_path', 'log_format']]
        )

        if prev_log_options != self._log_options and isinstance(self._logger, Logger):
            self._logger.update_logger(
                log_level=logging.getLevelName(self._config.get('log_level')),
                log_format=self._config.get('log_format'),
                log_path=self._config.get('log_path'),
                stream_out=self._config.get('debug'),
                coloredlog=self._config.get('debug'),
                log_when=self._config.get('log_when'),
                log_interval=self._config.get('log_interval'),
                log_backup_count=self._config.get('log_backup_count')
            )

    def write_config(self, config_path: Optional[Path] = None, encoding: str = 'utf-8'):
        """You can set config values in code, and save it as a file.

        :param config_path: Path or CONFIG_DEFAULT_FILE.
        :param encoding: file encoding.
        :return:
        """
        write_path = config_path or self._config_path
        with open(write_path, 'w', encoding=encoding) as config_file:
            json.dump(self._config, config_file, indent=4, sort_keys=True)

    def get_path(self, path: str) -> Path:
        """Get Path from the directory where the configure.json file is.

        :param path: file_name or path
        :return:
        """
        root_path = self._current_path or Path(os.path.join(os.getcwd()))
        return root_path.joinpath(path)



from pathlib import Path

import sys

from .config_ import Config
from .logger_ import Logger

__all__ = ['Config', 'Logger', 'config', 'logger']


def _get_main_module_name():
    _name = ''
    try:
        _name = sys.modules['__main__'].__file__
    except AttributeError:
        _name = sys.argv[0]

    return str(Path(_name).stem)


logger = Logger(name=_get_main_module_name())
config = Config(logger=logger)



"""Test Logger"""
import glob
import os
import time
from pathlib import Path

import pytest

from yirgachefe import logger


@pytest.fixture
def log_file() -> Path:
    root_path = Path(os.path.join(os.path.dirname(__file__), '.'))
    file_path: Path = root_path.joinpath('temp-log-file.log')
    return file_path


@pytest.fixture(autouse=True)
def clear_test_log_file(log_file):
    file_path: Path = log_file
    root_path = Path(os.path.join(file_path, '.'))

    file_list = glob.glob(f'{file_path}*')
    for file in file_list:
        path: Path = root_path.joinpath(file)
        if path.exists():
            path.unlink()

    yield

    file_list = glob.glob(f'{file_path}*')
    for file in file_list:
        path: Path = root_path.joinpath(file)
        if path.exists():
            path.unlink()


class TestLogger:
    def test_get_logger(self):
        logger.get_logger().warning('warning log')

    def test_autocomplete(self):
        logger.debug('debug log')

    def test_coloredlog(self):
        print()
        logger.update_logger(stream_out=True)
        logger.info('log')
        logger.critical('log')
        logger.error('log')
        logger.debug('log')
        logger.warning('log')
        logger.critical('log')

        logger.update_logger(stream_out=True, coloredlog=True)
        logger.info('color')
        logger.critical('color')
        logger.error('color')
        logger.debug('color')
        logger.warning('color')
        logger.critical('color')

    def test_log_file(self, log_file):
        print()
        logger.update_logger(log_path=str(log_file), stream_out=True)
        logger.debug('file')

        logger.update_logger(stream_out=True, coloredlog=True)
        logger.debug('color NO-FILE')

        logger.update_logger(log_path=str(log_file), stream_out=True, coloredlog=True)
        logger.debug('color file')

        with open(str(log_file), 'r') as _log_file:
            logs = _log_file.readlines()
            for log in logs:
                assert 'NO-FILE' not in log

    def test_log_rotating(self, log_file):
        print()
        logger.update_logger(log_path=str(log_file), stream_out=True, coloredlog=True,
                             log_when='s', log_interval=1, log_backup_count=5)
        for i in range(3):
            logger.debug('rotating file')
            time.sleep(1)

        file_path: Path = log_file
        root_path = Path(os.path.join(file_path, '.'))
        file_list = glob.glob(f'{file_path}*')
        for file in file_list:
            path: Path = root_path.joinpath(file)
            print(file)
            assert path.exists()

    def test_default_logger_name(self):
        from yirgachefe import _get_main_module_name
        assert logger.name == _get_main_module_name()



from pathlib import Path

from setuptools import setup, find_packages

req_tests = ["pytest"]
req_lint = ["flake8", "flake8-docstrings"]
req_dev = req_tests + req_lint

with open('requirements.txt', 'r') as f:
    install_requires = [
        s for s in [
            line.split('#', 1)[0].strip(' \t\n') for line in f
        ] if s != ''
    ]

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup_options = {
    "name": "yirgachefe",
    "version": "0.1.2",
    "url": "https://github.com/iconloop/Yirgachefe",
    "author": "ICONLOOP",
    "author_email": "t_core@iconloop.com",
    "description": "Convenience of configuration and logging.",
    "long_description": long_description,
    "long_description_content_type": "text/markdown",
    "package_data": {"yirgachefe": ["logger_.pyi"]},
    "packages": find_packages(),
    "python_requires": ">=3.9.7",
    "install_requires": install_requires,
    "extras_require": {
        "tests": req_tests,
        "lint": req_lint,
        "dev": req_dev
    },
    "package_dir": {"": "."},
}

setup(**setup_options)
